function r = sirene_rocanalysis(t, y, n)

% r = rocanalysis(t, y, n)
%
% Computes some statistics from a vector or true labels t (+/-1) and of
% predicted values y. n is the size of the vectors to be returned (the roc
% and pr curves are sampled over 1/n points between 0 and 1).
%
% The returned variable is a structure with the following fields:
% - roc : the ROC curve (vector of dimension n)
% - auc : area under the ROC curve
% - pr : the Precision-Recall (PR) curve (vector of dimension n)
% - aup : area under the PR curve
% - scorerecall : the score-recall curve (i.e., value of the score for
% different recall values between 0 and 1)

% preallocating the structure
r = struct('roc',[],'auc',[],'pr',[],'aup',[],'scorerecall',[]); 

% process targets
t = t > 0;

% sort by classifier output
[Y,idx] = sort(-y);
t       = t(idx);

% If everybody has the same label then we can not do much! Just return 0
if max(t)==min(t)
    r=0;
else
    
    % compute true positive and false positive rates

    tp = cumsum(t);
    fp = cumsum(~t);

    % roc
    largeroc = zeros(1,fp(end)+1);
    largeroc(fp+1)=tp/tp(end);
    r.roc = interp1([0:(1/fp(end)):1] , largeroc , [0:(1/(n-1)):1]);
    r.auc = mean(r.roc);

    % precision-recall
    largepr = zeros(1,(tp(end)+1));
    largepr(tp(end:-1:1)+1) = 1 - fp(end:-1:1) ./ (fp(end:-1:1)+tp(end:-1:1));
    largepr(1) = largepr(2);
    r.pr = interp1([0:(1/(tp(end))):1] , largepr , [0:(1/(n-1)):1]);
    r.aup = mean(r.pr);

    % score-recall
    largescore = zeros(1,(tp(end)+1));
    largescore(tp+1)=-Y;
    r.scorerecall = interp1([0:(1/(tp(end))):1] , largescore , [0:(1/(n-1)):1]);
end